
-- @file
-- The SQL portion of the 'Natural Sort' module.
--
-- It is important to read the "How does this module work" section
-- of the README.txt for better understanding of this code.

-- ---------------------------------------------------------------------------
-- The algorithm(s)
        
CREATE OR REPLACE FUNCTION natsort_canon(varchar, varchar) RETURNS varchar AS $FUNC$
DECLARE
  s          varchar  default $1;
  algorithm  varchar  default $2;
  ret        varchar  default '';
BEGIN

  IF s IS NULL THEN

    -- It is the custom in SQL to propagate NULL.
    RETURN NULL;

  ELSEIF NOT s ~ '[0-9]' THEN

    -- No numbers in this string, so skip the costly calculation.
    ret := s;

  ELSE

    -- Our task is to expand all numbers to have a fixed number of digits.
    -- 'I want 3.5 potatoes' -> 'I want [0000003500] potatoes'.

    -- Let's start by delimiting all numbers by the '~' character.
    -- 'I want 3.5 potatoes' -> 'I want ~3.5~ potatoes'.

    s := replace(s, '~', '');
    s := regexp_replace(s, $$([0-9]*\.?[0-9]+|[0-9][0-9,]+\.?[0-9,]*[0-9])$$, $$~\&~$$, 'g');

    DECLARE
      i         integer  default 1;
      part      varchar  default '';
      in_number boolean  default FALSE;
    BEGIN

      -- Now, let's split the string, on the '~' character, and loop over the elements.
      -- Odd elemenets are numbers. Even ones are texts.

      LOOP
        part := SPLIT_PART(s, '~', i);
        IF part = '' AND in_number THEN EXIT; END IF;
        IF in_number THEN
          -- I'm using 60 digits because if the user typed a long string of digits PgSQL will
          -- shout loudly.
          ret := ret || '[' || LPAD(CAST(REPLACE(part,',','') as decimal(60,3)), 15, '0') || ']';
        ELSE
          IF algorithm = 'firstnumber' AND ret = '' THEN
            -- if it's the 'firstnumber' algorithm and no number was encountered yet,
            -- then do nothing.
            NULL;
          ELSE
            ret := ret || part;
          END IF;
        END IF;   
        in_number := NOT in_number;
        i := i + 1;
      END LOOP;
    END;
    
  END IF;

  -- I don't know how much this is needed, but let's remove all spaces
  -- and some punctuation marks.
  ret := TRANSLATE(ret, ' ,:.;()', '');
  RETURN ret;

END;
$FUNC$ LANGUAGE plpgsql;

-- ---------------------------------------------------------------------------
-- Utilities

-- natsort_canon_save() is just like natsort_canon() except it saves the
-- calculation result for next time. See README.txt for info on how to use it.
CREATE OR REPLACE FUNCTION natsort_canon_save(varchar, varchar) RETURNS varchar AS $FUNC$
BEGIN
  RETURN natsort_canon($1, $2);
END;
$FUNC$ LANGUAGE plpgsql;

-- Type 'select natsort_benchmark();' at the prompt for time estimation.
CREATE OR REPLACE FUNCTION natsort_benchmark() RETURNS void AS $FUNC$
BEGIN
  FOR i in 1..1000 LOOP
    PERFORM natsort_canon('some testing, 123 blah', 'natural');
  END LOOP;
END;
$FUNC$ LANGUAGE plpgsql;

-- ---------------------------------------------------------------------------
-- Initializer and finalizer.
--
-- These are called by Drupal to initialize and stop the system.

CREATE OR REPLACE FUNCTION natsort_initialize() RETURNS void AS $FUNC$
BEGIN
  NULL;
END;
$FUNC$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION natsort_finalize() RETURNS void AS $FUNC$
BEGIN
  NULL;
END;
$FUNC$ LANGUAGE plpgsql;

-- ---------------------------------------------------------------------------
-- Tables

-- A table to cache all calculated values.
DROP TABLE IF EXISTS natsort_lookup;
CREATE TABLE natsort_lookup (
  "algorithm" varchar(20)  NOT NULL default '',
  "source"    varchar(255) NOT NULL default '',
  "target"    varchar(255) NOT NULL default '',
  PRIMARY KEY ("algorithm", "source")
);

